--[[
	test (parts of) the lua scripting functionality
--]]

local _M = {}

function _M.basic_test(context)
	log("basic_test: this function has been called from within context %s", context)
end

function _M.pkg_test(package)
	local lst = pkg.list(package)
	log("pkg_test: the package %s contains %d entries\nThe first 10 are:", package, #lst)
	for i = 1, 10 do
		log("\t%s", lst[i])
	end
end

function _M.config_test()
	local numdlc = config.getvar("NumDLC")
	if numdlc ~= 1 then
		log("config_test: failed, wrong numdlc")
		return
	end
	local dlc0 = config.getvar("DLC0")
	if dlc0 == nil then
		log("config_test: failed, wrong DLC0")
		return
	end
	local pkg = config.getvar("PackageFilename", dlc0)
	log("config_test: result 1 is %s", pkg)

	config.setvar("Hauptgang", dlc0, "Wurstsalat")
	local var = config.getvar("Hauptgang", dlc0)
	log("config_test: result 2 is %s", var)

	local var2 = config.getvar("Test", "LuaTest")
	log("config_test: result 3 is %s: %s", var2, (var2 == config.hash("Test")) and "ok" or "nok" )	
end

-- dynamic texture generation test
function _M.texturegen_test()
	local t = texture.load("Textures/UI/cheater_NODXT.tga")
	if t == nil then
		log("texturegen_test: failed, could not load texture")
		return
	end

	for x = 0, t.sizex, 2 do
		for y = 0, t.sizey, 2 do
			t:setpixel(x, y, 255, 0, 0, 255)
		end
	end

	local s = stream.create()
	t:saveto(s)
	return s
end

-- change library-n.eldritchroom to dynamic generation
function _M.patch_library_n()
	local nroomdefs = config.getvar("NumRoomDefs", "WorldGenDef_Library")
	local which
	for i=0, nroomdefs-1 do
		local roomdef = config.getvar("RoomDef" .. i, "WorldGenDef_Library")
		if string.find(roomdef, "/library-n.eldritchroom", 1, true) then
			which = i
			break
		end
	end
	config.setvar("RoomDef" .. which, "WorldGenDef_Library", "&LuaTest.luatest.roomgen_test('/library-n.eldritchroom')")
	log("changed library-n to %s", config.getvar("RoomDef" .. which, "WorldGenDef_Library"))
end

-- dynamic room generation test
function _M.roomgen_test(name)
	print("roomgen name: "..name)
	local r = room.create()
	-- generates a checkered pattern on the west, south and east sides of the room
	for x=0, r.sizex-1 do
		for z=0, r.sizez-1 do
			if (x + z) & 1 ~= 0 then
				r:setvoxel(x, 0, z, 1)
			end
		end
	end
	for y=0, r.sizey-1 do
		for z=0, r.sizez-1 do
			if (y + z) & 1 ~= 0 then
				r:setvoxel(0, y, z, 1)
			end
			if (y + z) & 1 == 0 then
				r:setvoxel(r.sizex-1, y, z, 1)
			end
		end
	end
	local s = stream.create()
	r:saveto(s)
	return s
end

return _M
